# Generated by Django 5.2.5 on 2025-10-03 20:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isapilib', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='apilogs',
            name='fecharegistro',
            field=models.DateTimeField(),
        ),
    ]
