# Generated by Django 5.2.8 on 2025-12-03 14:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isapilib', '0002_alter_apilogs_fecharegistro'),
    ]

    operations = [
        migrations.CreateModel(
            name='CuentasPorCobrar',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('mov', models.CharField(db_column='Mov', max_length=20)),
                ('mov_id', models.CharField(blank=True, db_column='MovID', max_length=20, null=True)),
            ],
            options={
                'db_table': 'Cxc',
                'managed': False,
            },
        ),
        migrations.AlterModelOptions(
            name='apilogs',
            options={'ordering': ['-fecharegistro', 'id']},
        ),
        migrations.AlterField(
            model_name='branchapi',
            name='gwmbac',
            field=models.CharField(max_length=100, null=True, unique=True),
        ),
    ]
