# Generated by Django 5.2.5 on 2025-12-04 20:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isapilib', '0003_cuentasporcobrar_alter_apilogs_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='connectionapi',
            name='connection_retries',
            field=models.IntegerField(default=5),
        ),
        migrations.AddField(
            model_name='connectionapi',
            name='connection_retry_backoff_time',
            field=models.IntegerField(default=5),
        ),
        migrations.AddField(
            model_name='connectionapi',
            name='connection_timeout',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='connectionapi',
            name='query_timeout',
            field=models.IntegerField(default=0),
        ),
    ]
